from inc_noesis import *


def registerNoesisTypes():
	handle = noesis.register("Kill la Kill IF", ".mdl")
	noesis.setHandlerTypeCheck(handle, noepyCheckType)
	noesis.setHandlerLoadModel(handle, noepyLoadModel)
	return 1

def noepyCheckType(data):
	if len(data) < 8:
		return 0
	bs = NoeBitStream(data)

	if bs.readInt() != int(1179468876):
		return 0

	return 1

class Material():
    def __init__(self, *args, **kwargs):
        self.Name = str()
        self.FaceStart = int()
        self.FaceSize = int()
        self.FaceCount = int()
        self.FaceType = int()
        return super().__init__(*args, **kwargs)


def noepyLoadModel(data, mdlList):
    bs = NoeBitStream(data)
    ctx = rapi.rpgCreateContext()
    Skin = 1
    
    ID = bs.readUInt()
    ModelNameLOC = bs.readUInt()
    MATNameLOC = bs.readUInt()
    MATCount = bs.readUInt()
    BoneLOC = bs.readUInt()
    BoneCount = bs.readUInt()
    VertBuffLOC = bs.readUInt()
    VertBuffSize = bs.readUInt()
    VertexCount = bs.readUInt()
    VertStride = int(VertBuffSize / VertexCount)
    TotalFaceCount = bs.readUInt()
    ModelName = bs.readBytes(64).decode("ASCII").rstrip("\0")
    print(ModelName)
    BoneList = []

    MatList = []
    FaceBuffers = []
    bs.seek(MATNameLOC)
    for X in range(0, MATCount):
        M = Material()
        M.Name = bs.readBytes(64).decode("ASCII").rstrip("\0")
        M.FaceStart = bs.readUInt()
        M.FaceSize = bs.readUInt()
        M.FaceCount = bs.readUInt()
        M.FaceType = bs.readUInt()
        bs.seek(96, NOESEEK_REL)
        MatList.append(M)
    for X in range(0, MATCount):
        U = MatList[X]
        bs.seek(U.FaceStart)
        FaceBuffers.append(bs.readBytes(U.FaceSize))

    if BoneLOC:
        bs.seek(BoneLOC)
        for B in range(0, BoneCount):
            BoneName = bs.readBytes(24).decode("ASCII").rstrip("\0")
            UNK = bs.readBytes(8)
            BoneROT = NoeQuat.fromBytes(bs.readBytes(16)).transpose()
            BonePOS = NoeVec3.fromBytes(bs.readBytes(12))
            BoneSCA = NoeVec3.fromBytes(bs.readBytes(12))
            BoneParent = bs.readInt()
            BoneROT = BoneROT.toMat43()
            BoneROT[3] = [BonePOS[0], BonePOS[1], BonePOS[2]]
            BoneROT.__mul__(BoneSCA)
            BoneList.append(NoeBone(B, BoneName, BoneROT, None, BoneParent))

    bs.seek(VertBuffLOC)
    VertexBuff = bs.readBytes(VertBuffSize)
    rapi.rpgBindPositionBufferOfs(VertexBuff, noesis.RPGEODATA_FLOAT, VertStride, 0)
    rapi.rpgBindNormalBufferOfs(VertexBuff, noesis.RPGEODATA_HALFFLOAT,  VertStride, 16)
    rapi.rpgBindUV1BufferOfs(VertexBuff, noesis.RPGEODATA_HALFFLOAT,  VertStride, 24)
    rapi.rpgBindColorBufferOfs(VertexBuff, noesis.RPGEODATA_UBYTE,  VertStride, 12, 4)
    if len(BoneList) >= 1:
        if Skin:
            print("Exporting Skin")
            if VertStride == 48:
                rapi.rpgBindBoneIndexBufferOfs(VertexBuff, noesis.RPGEODATA_USHORT, 48, 36, 4)
                rapi.rpgBindBoneWeightBufferOfs(VertexBuff, noesis.RPGEODATA_UBYTE, 48, 44, 4)
            elif VertStride == 40:
                rapi.rpgBindBoneIndexBufferOfs(VertexBuff, noesis.RPGEODATA_USHORT, 40, 28, 4)
                rapi.rpgBindBoneWeightBufferOfs(VertexBuff, noesis.RPGEODATA_UBYTE, 40, 36, 4)
            elif VertStride == 60:
                rapi.rpgBindBoneIndexBufferOfs(VertexBuff, noesis.RPGEODATA_USHORT, 60, 36, 4)
                rapi.rpgBindBoneWeightBufferOfs(VertexBuff, noesis.RPGEODATA_FLOAT, 60, 44, 4)
            else:
                print("Unknown VertStride", VertStride)
    for t in range(0, len(MatList)):
        q = MatList[t]
        rapi.rpgSetMaterial(q.Name)
        rapi.rpgSetName(ModelName+"_"+str(t))
        if q.FaceType == int(2):
            rapi.rpgCommitTriangles(FaceBuffers[t], noesis.RPGEODATA_USHORT, q.FaceCount, noesis.RPGEO_TRIANGLE, 1)
        elif q.FaceType == int(4):
            rapi.rpgCommitTriangles(FaceBuffers[t], noesis.RPGEODATA_UINT, q.FaceCount, noesis.RPGEO_TRIANGLE, 1)
        else:
            print("Unknown Face Type", print(q.FaceType))
    mdl = rapi.rpgConstructModel()
    rapi.rpgClearBufferBinds()
    if len(BoneList) >= 1:
        mdl.setBones(BoneList)
    mdlList.append(mdl)


    return 1
